.na
.nh
.TH transferfid 1 2.6.13 BL3DEMC
.SH NAME
transferfid \- make seed model for second axis from fiducials for first axis
.SH SYNOPSIS
transferfid [options] setname
.SH DESCRIPTION
transferfid will make a seed model for tracking fiducials in one axis 
of a dual-axis tilt series, using the fiducial model for the other axis.  It
first finds a pair of views from the two axes that match up well after
an appropriate translation, rotation, and stretching of the view from the
first axis.  It then transforms the corresponding section of the fiducial
model from the first axis by the same amount, and uses Beadtrack to find each
bead on the view from the second axis.  This collection of new positions then
forms the seed model for tracking the second axis.

The program will do all of these operations without any arguments except the
name of the data set if all files
are named appropriately; specifically, if
the command files for beadtrack are named tracka.com and
trackb.com, and the tilt
angles are in files with extension .rawtlt.  
By default, the program will transfer fiducials from the "a" axis
to the "b" axis, in which case it assumes the first fiducial model is named 
setnamea.fid.  However, you can use the 
.B -b
option to transfer from the "b" axis to the "a" axis, in which case the 
existing fiducial model is assumed to be named setnameb.fid.
By default, the program will
search for the best matching pair of views using the 5 views nearest to zero
tilt from each data set.  It will consider rotations by both -90 and +90
degrees, and it will set the bead radius for Beadtrack using the value
in tracka.com.

If the program fails to find the proper alignment between the two axes, this
is probably because the rotation is too far from 90 degrees, and/or the 
translations are too large. 
You can solve this problem by using the 
.B -m
option to
run midas(1) and set the initial rotational
and translational alignment manually.  
When you get into Midas, you may need to rotate
the image by 180 degrees, unless you specify the angle in advance with the
.B -a
option, but this large rotation will not cause problems.  If you find that
the images are mirror images of each other, then you can either mirror one
around the X axis in Midas using the entry in the Edit menu, or
go back to running Transferfid without Midas but with the 
.B
-x
option.

Once the seed model is created, the program analyzes contours in the fiducial
model from the first axis to provide information that is needed when
running solvematch(1).  If the 
.B -c
option is used, then corresponding coordinates of the transferred beads
in the two data sets will be placed into the specified
output file.  This information is sufficient for solvematch(1) to identify
the corresponding fiducials in the final alignment, regardless of whether
some seed points fail to track or some contours are deleted from either 
fiducial model.  If this option is not
used, the program indicates what the correspondence between points
should be when running solvematch(1).  The list of points in A could be in 
error in some cases unless a fiducial coordinate file (setnameafid.xyz) is
also available.  For this reason, it is advisable to run tiltalign(1) at least
once on the first fiducial model before running transferfid.  The lists could
also become erroneous if one of the points in the new seed model fails to 
track, or if fiducials are deleted from either model after running transferfid.
If such changes occur, just enter a small starting subset of points and let
solvematch(1) come up with the rest of the correspondences.

Finding the best pair of views is somewhat time-consuming.  Two ways to
reduce this time are to reduce the number of views in the search with the
.B -n
option and to specify the direction of rotation between the axes with the
.B -a
option.  The latter option alone will save only 15% of the time, because
the program will abandon one rotation direction if it is clearly worse after
considering four pairs of views.

.SS Options
.TP
.B -b
Transfer backwards, from the "b" axis to the "a" axis.  You will generally
not need to specify any filenames with this option.
.TP
.B -ia <image file>
Set the name of the image file upon which the fiducial model for the "a" axis
tilt series was built; the default is to take the name from the tracka.com
command file.
.TP
.B -ib <image file>
Set the name of the image file for tracking beads in the "b" axis tilt series;
the default is to take the name from the trackb.com command file.
.TP
.B -f <model file>
Set the name of the fiducial model file for the first, already tracked tilt 
series; the default is setnamea.fid (or setnameb.fid with the 
.B -b
option.)
.TP
.B -o <model file>
Set the name of the output file for the seed model; the default is
setnameb.seed (or setnamea.fid with the
.B -b
option.)
.TP
.B -n <value>
Set the number of views from each data set to include in the search for the
best matching pair of views; the default is 5.
.TP
.B -za <value>
Set the number of the view from the "a" axis tilt series around which to search
for the best match; the default is the view nearest to zero tilt, using
tilt angles in the file setnamea.rawtlt.
.TP
.B -zb <value>
Set the number of the view from the "b" axis tilt series around which to search
for the best match; the default is the view nearest to zero tilt, using
tilt angles in the file setnameb.rawtlt.
.TP
.B -a <value>
Use this option to specify the direction of rotation from the first to the
second axis.  Only the sign of this number matters, so you can enter -1 or -90
for clockwise rotations, or 1 or 90 for counterclockwise rotations.  In the
absence of this option, the program will consider both directions of rotation.
.TP
.B -x
Mirror one image around the X axis before rotating by 90 degrees.
.TP
.B -m
Run Midas on the first pair of views, and use the alignment that you set in
Midas as the initial alignment for all pairs of views that are considered.
.TP
.B -d <value>
Set the value of the bead diameter in pixels in the images being used; the
default is to take the value from tracka.com (or trackb.com with the
.B -b
option.)  Specifically, if a radius for centroid is found there it is 
converted to a diameter by 2 * radius - 3; if a bead diameter is found it
is treated as a diameter in unbinned pixels and the image binning is also
extracted from tracka.com.
.TP
.B -l
Use this option to specify that beads are light on a dark background.  It has
an effect only if the 
.B -d
option is used; otherwise this parameter is taken from tracka.com (or 
trackb.com with the
.B -b
option.)
.TP
.B -c <file name>
Use this option to have corresponding fiducial coordinates on the best matching
views placed into an output file.  This file will also record the section
numbers in the two sets and whether the transfer was from "b" to "a".
.TP 
.B -t
Leave temporary files in /usr/tmp for diagnosing problems.
.TP 
.B -P
Output process ID.
.SH FILES
Whether or not they fit the default naming conventions, the image files for
fiducial tracking for each tilt series and the first fiducial model
must be present in the current directory.
If the output file already exists, a backup image
of the original is created
with the ~ extension.
.SH AUTHOR
David Mastronarde  <mast@colorado.edu>
.SH SEE ALSO
beadtrack(1), tiltxcorr(1), remapmodel(1), clipmodel(1), xfsimplex(1),
xfmodel(1), midas(1), solvematch(1), tiltalign(1)
